import pytest

from amphipathic.hydrophobic import cornette

EXPECTED_SCALES = [
    'zimmr', 'ntan', 'ntanr', 'jones', 'levit', 'hoppw', 'yungd', 'faupl',
    'zaslz', 'wolf', 'kuntz', 'abodr', 'meek', 'buldg', 'eisen', 'kytdo',
    'choth', 'wersc', 'janin', 'olsen', 'meiro', 'ponnu', 'nneig', 'robos',
    'chdlg', 'wsdlg', 'jadlg', 'guy', 'guym', 'kridg', 'krigk', 'nioii',
    'mijer', 'rosef', 'sweet', 'sweig', 'rekkr', 'vhebl', 'fromm', 'eimcl',
    'prift', 'prils', 'altft', 'altls', 'totft', 'totls',
]


def test_get_scales():
    assert cornette.get_scales() == EXPECTED_SCALES


EXPECTED_CLEANED_VALUES = [
    ("1.1", 1.1),
    ("-2.1", -2.1),
    ("N", "N"),
    ("TRUE", True),
    ("FALSE", False),
    ("invalid", "invalid"),
]


@pytest.mark.parametrize("value,expected", EXPECTED_CLEANED_VALUES)
def test_clean(value, expected):
    assert cornette.clean(value) == expected


EXPECTED_PROPERTIES_VALUES = [
    (
        cornette.is_experimental,
        True,
        "prift",
        False
    ),
    (
        cornette.is_experimental,
        False,
        "prift",
        False
    ),
    (
        cornette.is_owned,
        True,
        "prift",
        True
    ),
    (
        cornette.is_average,
        True,
        "prift",
        False
    ),
    (
        cornette.is_statistical,
        True,
        "prift",
        True
    ),
    (
        cornette.is_statistical,
        True,
        "ponnu",
        True
    )
]


@pytest.mark.parametrize("property,normalized,scale,expected", EXPECTED_PROPERTIES_VALUES)
def test_properties(property,normalized,scale,expected):
    assert property(normalized=normalized, scale=scale) == expected


def test_load_table_unknown_scale():
    with pytest.raises(cornette.UnknownScaleError):
        cornette.load_table(normalized=True, scale="invalid_one")



# -*- coding: utf-8 -*- #
"""Core Service."""
from __future__ import unicode_literals

import re
import warnings
from math import cos, sin

from Bio import BiopythonWarning, Seq

from amphipathic import secondary_structure
from amphipathic.hydrophobic.core import select_table as select_hydrophobic_table


def is_nucleotide(characters):
    return all([p in ['a', 't', 'c', 'g'] for p in characters])


def obtain_structure(params):
    primary, secondary = params
    structure = []
    elements = [
        m.group(0)
        for m in re.finditer(r"(\w)\1*", secondary)
    ]
    idx = 0
    for e in elements:
        s = {'type': e[0],
             'begin': idx,
             'end': idx + len(e),
             'seq': primary[idx:idx + len(e)]}
        structure.append(s)
        idx += len(e)
    return structure


def apply_secondary_structure(primary):
    return [
        secondary_structure.jpred(seq)
        for seq in primary
    ]


class Sequence(object):

    def __init__(self, string):
        string = string.lower()
        if is_nucleotide(string):
            self.nucleotide = string
            warnings.simplefilter('ignore', BiopythonWarning)
            string = Seq.translate(string).lower()
        self.primary = string.split('*')
        self.secondary = []
        self.structures = []

    def resume_secondary(self):
        if not self.structures:
            self.secondary = apply_secondary_structure(self.primary)
            self.structures = [
                obtain_structure(p)
                for p in zip(self.primary, self.secondary)
            ]
            self.structures = [s for s in self.structures if s]
        return self.structures


FGR = 0.0174533


def apply_func(func, i, h, mean, coefficient):
    return (h - mean) * func(i * coefficient * FGR)


def partial_sum(amph, mean, coefficient):
    sum1 = sum([
        apply_func(cos, i, h, mean, coefficient) for i,h in enumerate(amph)
    ])
    sum2 = sum([
        apply_func(sin, i, h, mean, coefficient) for i, h in enumerate(amph)
    ])
    return sum1 ** 2 + sum2 ** 2


def total_sum_norm(h, mean, begin, end, step):
    diff = end - begin
    m1 = 1. / (diff * FGR)
    f1 = ((end * FGR - begin * FGR) / diff * FGR) / 2.
    sumparc1 = partial_sum(h, mean, begin)
    sumparc2 = sum([
        2 * partial_sum(h, mean, angle)
        for angle in range(begin, end, step)
    ])
    sumparc3 = partial_sum(h, mean, end)
    sumtot = sumparc1 + sumparc2 + sumparc3
    return m1 * (f1 * sumtot)


def calculate_amphipathic_index(struct, **kwargs):
    angles = {
        'c': (1, 180),
        'e': (100, 160),
        'h': (80, 120),
    }
    seq = struct['seq']
    hydrophobic_table = select_hydrophobic_table(**kwargs)
    hydrophobic_scores = [
        hydrophobic_table[aa]
        for aa in seq
        if aa in hydrophobic_table
    ]
    amount = len(hydrophobic_scores)
    amount = amount if amount > 0 else 1.
    mean = sum(hydrophobic_scores) / amount
    begin, end = angles[struct['type']]
    step = 1  # 1 degree by step
    num = total_sum_norm(hydrophobic_scores, mean, begin, end, step)
    den = total_sum_norm(hydrophobic_scores, mean, 1, 180, 1)
    ai = num / (den if den != 0 else 1)
    return ai, mean


def add_indexes_to_secuence(struct, **kwargs):
    index, mean = calculate_amphipathic_index(struct, **kwargs)
    struct.update(dict(
        amphipathic=dict(
            index=index,
            mean=mean
        )
    ))
    return struct


def index_secondary(seq, **kwargs):
    [
        [
            add_indexes_to_secuence(aa, **kwargs)
            for aa in protein
        ]
        for protein in seq.resume_secondary()
    ]


def index(sequence, **kwargs):
    seq = Sequence(sequence)
    seq.resume_secondary()
    index_secondary(seq, **kwargs)
    return seq.resume_secondary()



import csv
from functools import lru_cache
import os
import amphipathic


class UnknownScaleError(Exception):
    pass


@lru_cache(maxsize=None)
def load(filename):
    complete_path = os.path.join(amphipathic.__path__[0], "hydrophobic", "data", filename)
    with open(complete_path, newline='', mode="r") as csvfile:
        reader = csv.reader(csvfile)
        return {
            row[0]: row[1:]
            for row in reader
        }


ACCEPTED_VALUES = {
    "TRUE": True,
    "FALSE": False,
}


def clean(value):
    try:
        return float(value)
    except ValueError:
        return ACCEPTED_VALUES.get(value, value)


@lru_cache(maxsize=None)
def load_file(normalized:bool):
    filename = "cornette"
    if normalized:
        filename += "_normalized"
    filename += ".csv"
    return load(filename)


@lru_cache(maxsize=None)
def load_table(normalized:bool, scale:str):
    db = load_file(normalized)
    headers = [head.lower() for head in db["name"]]
    try:
        values = db[scale.lower()]
    except KeyError:
        raise UnknownScaleError
    live_table = {
        key: clean(value)
        for (key, value) in zip(headers, values)
    }
    return live_table


@lru_cache(maxsize=None)
def get_scales(normalized=True):
    db = load_file(normalized)
    return list(db.keys())[1:]


@lru_cache(maxsize=None)
def select_table(normalized=True, scale="PRIFT"):
    table = load_table(normalized, scale)
    return {
        key: value
        for (key, value) in table.items()
        if len(key) == 1
    }


@lru_cache(maxsize=None)
def get_property(normalized=True, scale="PRIFT", property="experimental"):
    table = load_table(normalized, scale)
    return table[property]


def is_experimental(normalized=True, scale="PRIFT"):
    return get_property(normalized, scale, "experimental")


def is_statistical(normalized=True, scale="PRIFT"):
    return get_property(normalized, scale, "statistic")


def is_average(normalized=True, scale="PRIFT"):
    return get_property(normalized, scale, "average")


def is_owned(normalized=True, scale="PRIFT"):
    return get_property(normalized, scale, "own")



# -*- coding: utf-8 -*- #
"""Secondary Structure Services."""
from __future__ import unicode_literals

import re
import time

import requests
from bs4 import BeautifulSoup
from cachetools.func import ttl_cache


class AmphipathicSecondaryStructureError(Exception):
    pass


@ttl_cache(ttl=10*60)
def gor(seq):
    # It uses the GOR4 service to estimate the secondary structure.
    data = {
        'title': '',
        'notice': seq,
        'ali_width': len(seq)
    }
    html = BeautifulSoup(requests.post(
        'https://npsa-prabi.ibcp.fr/cgi-bin/secpred_gor4.pl',
        data).text, 'html.parser')
    return ''.join([
        f.text
        for f in html.select('code font')
    ])


@ttl_cache(ttl=10*60)
def jpred(seq):
    url = 'http://www.compbio.dundee.ac.uk/jpred4/cgi-bin/rest/job'
    separator = '£€£€'
    params = [
        'skipPDB=on',
        'format=seq',
        '>query\n' + seq
    ]
    data = separator.join(params)
    headers = {
        #'TE': 'deflate, gzip;q = 0.3',
        #'Connection': 'TE, close',
        'User-Agent': 'libwww-perl/6.38',
        'Content-Type': 'text/txt',
    }
    res = requests.post(
        url,
        data.encode('utf-8'),
        headers=headers
    )
    if res.status_code == 202:
        matches = re.search(
            r'(?:href=[\'"])([:/.A-z?<_&\s=>0-9;-]+)',
            res.content.decode('utf-8')
        )
        if not matches:
            # The services doesn't accept the sequence.
            return 'c' * len(seq)
        job_link = matches.group(1)
        job_id = job_link.split('?')[1]
        url = 'http://www.compbio.dundee.ac.uk/jpred4/results/{job_id}/{job_id}.jnet'.format(job_id=job_id)
        res = requests.get(url)
        while res.status_code == 404:
            time.sleep(10)
            res = requests.get(url)
        data = res.content.decode('utf-8')
        return data.split('\n')[1].replace('jnetpred:', '').replace(',', '').lower().replace('-', 'c')
    else:
        raise AmphipathicSecondaryStructureError('We cannot predict the secondary structure')



import pytest

@pytest.fixture(name="sequence0")
def fixture_sequence_0():
    return 'NLYIQWLXDGGPSSGRPPPS'


@pytest.fixture(name="sequence1")
def fixture_sequence_1():
    return 'NLYIQWLKDGGPSSGRPPPS'


@pytest.fixture(name="sequence2")
def fixture_sequence_2():
    return 'MALPVTALLLPLALLLHAARPSQFRVSPLDRTWNLGETVELKCQVLLSNPTSGCSWLFQPRGA'


@pytest.fixture(name="sequence3")
def fixture_sequence_3():
    return 'ggctgacggctgggttaagaagtttactgaggcagttaacgcctttaagggtttggactggatc'\
           'cgctgccaagttgtccaagttccttgattggatcaaatccaagatcatcccggagctcagagag'\
           'agagcggagtttgttaagaatcttaggcagcttcctcttctcgaggcccagatcaccactttgg'\
           'agcactccaaccctaatcaggagacccaagaacagcttttctcgaacgtccaatacctggcaca'\
           'ccactgtaggaagaacgctccgctctatgcagcggaagcccggagagttttcgcactagagaaa'



import pytest
import os
import sys
import json


@pytest.fixture(scope="class")
def load_input():
    def loader(filename):
        path = os.path.dirname(os.path.abspath(__file__))
        with open('{:}/input-samples/{:}'.format(path, filename), 'r') as f:
            data = f.read()
        return unicode(data) if sys.version_info[0] == 2 else data
    return loader


@pytest.fixture(scope="class")
def load_json():
    def loader(filename):
        path = os.path.dirname(os.path.abspath(__file__))
        with open('{:}/input-samples/{:}'.format(path, filename), 'r') as f:
            data = json.load(f)
        return data
    return loader



# -*- coding: utf-8 -*- #
"""Secondary Structure Services."""
from __future__ import unicode_literals


import amphipathic


def test_amphipathic_index_0(sequence0, mocker):
    mocker.patch.object(amphipathic, 'apply_secondary_structure')
    amphipathic.apply_secondary_structure.return_value = [
        'ccceehhhcccccccccccc'
    ]
    resume = amphipathic.index(sequence0)
    assert resume[0] == [
        {
            'amphipathic': {'index': 1.0, 'mean': 2.81},
            'begin': 0,
            'end': 3,
            'seq': 'nly',
            'type': 'c'
        },
        {
            'amphipathic': {'index': 1.6242872515167734, 'mean': 0.9799999999999998},
             'begin': 3,
             'end': 5,
             'seq': 'iq',
             'type': 'e'
        },
        {
            'amphipathic': {'index': 1.1841818371579635, 'mean': 3.35},
             'begin': 5,
             'end': 8,
             'seq': 'wlx',
             'type': 'h'
        },
        {
            'amphipathic': {'index': 1.0, 'mean': -0.9941666666666668},
            'begin': 8,
            'end': 20,
            'seq': 'dggpssgrppps',
            'type': 'c'
        }
    ]


def test_amphipathic_index_1(sequence1, mocker):
    mocker.patch.object(amphipathic, 'apply_secondary_structure')
    amphipathic.apply_secondary_structure.return_value = [
        'ccceehhhcccccccccccc'
    ]
    resume = amphipathic.index(sequence1)
    assert resume[0] == [
        {
            'type': 'c',
            'begin': 0,
            'end': 3,
            'seq': 'nly',
            'amphipathic': {'index': 1.0, 'mean': 2.81}
        },
        {
            'type': 'e',
            'begin': 3,
            'end': 5,
            'seq': 'iq',
            'amphipathic': {'index': 1.6242872515167734, 'mean': 0.9799999999999998}
        },
        {
            'type': 'h',
            'begin': 5,
            'end': 8,
            'seq': 'wlk',
            'amphipathic': {'index': 1.155091623405534, 'mean': 1.22}
        },
        {
            'type': 'c',
            'begin': 8,
            'end': 20,
            'seq': 'dggpssgrppps',
            'amphipathic': {'index': 1.0, 'mean': -0.9941666666666668}
        }
    ]


def test_amphipathic_index_2(sequence2, mocker):
    mocker.patch.object(amphipathic, 'apply_secondary_structure')
    amphipathic.apply_secondary_structure.return_value = [
        'ccchhhhhhhhhhhhhhhhccccceeccccccccccceeeeeeeeecccccccceeeeccccc'
    ]
    resume = amphipathic.index(sequence2, scale="prift")
    assert resume[0] == [
        {
            'type': 'c',
            'begin': 0,
            'end': 3,
            'seq': 'mal',
            'amphipathic': {'index': 1.0, 'mean': 3.3699999999999997}
        },
        {
            'type': 'h',
            'begin': 3,
            'end': 19,
            'seq': 'pvtalllplalllhaa',
            'amphipathic': {'index': 0.7203991559243234, 'mean': 2.454375}
        },
        {
            'type': 'c',
            'begin': 19,
            'end': 24,
            'seq': 'rpsqf',
            'amphipathic': {'index': 1.0, 'mean': 0.07400000000000002}
        },
        {
            'type': 'e',
            'begin': 24,
            'end': 26,
            'seq': 'rv',
            'amphipathic': {'index': 1.6242872515167737, 'mean': 3.045}
        },
        {
            'type': 'c',
            'begin': 26,
            'end': 37,
            'seq': 'spldrtwnlge',
            'amphipathic': {'index': 1.0, 'mean': 0.35000000000000003}
        },
        {
            'type': 'e',
            'begin': 37,
            'end': 46,
            'seq': 'tvelkcqvl',
            'amphipathic': {'index': 0.9132957541599984, 'mean': 1.6855555555555555}
        },
        {
            'type': 'c',
            'begin': 46,
            'end': 54,
            'seq': 'lsnptsgc', 'amphipathic': {'index': 1.0, 'mean': 0.53}
        },
        {
            'type': 'e',
            'begin': 54,
            'end': 58,
            'seq': 'swlf',
            'amphipathic': {'index': 0.6423054796591475, 'mean': 2.6725000000000003}
        },
        {
            'type': 'c',
            'begin': 58,
            'end': 63,
            'seq': 'qprga',
            'amphipathic': {'index': 1.0, 'mean': -0.6799999999999999}
        }
    ]


def test_amphipathic_index_3(sequence3, mocker):
    mocker.patch.object(amphipathic, 'apply_secondary_structure')
    amphipathic.apply_secondary_structure.return_value = [
        'c',
        'ccc',
        'ccc',
        'cc',
        'cc',
        'ccccccccccccccccccccccccchhhhhhhhhhhhhhhcccccccccccccccccccccccccccchhhhhhhhhhhccccccccc',
        'c'
    ]
    resume = amphipathic.index(sequence3)
    for prot in resume:
        for struct in prot:
            assert 'amphipathic' in struct
    seq = [
        [
            s for s in prot
            if s['amphipathic']['index'] > 2.5
        ]
        for prot in resume
    ]
    print("---")
    print(filter(lambda protein: protein, seq))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from setuptools import setup
import os
import subprocess
from setuptools.command import easy_install


__version__ = '1.0.8'


def parse_requirements(filename):
    return [
        line.strip()
        for line in open(filename).readlines()
        if (line.strip())[0] != '#'
    ]


requirements = parse_requirements('requirements.txt')


def get_long_description():
    readme_file = 'README.md'
    if not os.path.isfile(readme_file):
        return ''
    # Try to transform the README from Markdown to reStructuredText.
    try:
        easy_install.main(['-U', 'pyandoc==0.0.1'])
        import pandoc
        pandoc.core.PANDOC_PATH = 'pandoc'
        doc = pandoc.Document()
        doc.markdown = open(readme_file).read()
        description = doc.rst
    except Exception:
        description = open(readme_file).read()
    return description


setup(
    name='amphipathic',
    version=__version__,
    author=u'Eloy Adonis Colell',
    author_email='eloy.colell@gmail.com',
    packages=['amphipathic', 'amphipathic.hydrophobic', 'amphipathic.hydrophobic.data'],
    url='https://github.com/ecolell/amphipathic',
    license='MIT',
    description=('This is a library to evaluate an aminoacid sequence and '
                 'determine an amphipathic index for each alpha helix or '
                 'beta sheet.'),
    long_description=get_long_description(),
    long_description_content_type="text/markdown",
    zip_safe=True,
    install_requires=requirements,
    classifiers=[
        "Intended Audience :: Science/Research",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.6",
        "Topic :: Scientific/Engineering :: Bio-Informatics",
        "Topic :: Scientific/Engineering :: Information Analysis",
        "Topic :: Scientific/Engineering :: Physics",
    ],
    include_package_data=True,
)



import enum
from typing import Dict
from amphipathic.hydrophobic import cornette


class Author(enum.Enum):
    CORNETTE = "cornette"


def select_table(author:Author=Author.CORNETTE, **kwargs) -> Dict[str, float]:
    table = {}
    if author == Author.CORNETTE:
        table = cornette.select_table(**kwargs)
    return table
